@@ -4,7 +4,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'AR': 'ar',
  'ARFLAGS': 'rc',
  'ATHEOS_THREADS': 0,
- 'BASECFLAGS': '-fno-strict-aliasing -OPT:Olimit=0',
+ 'BASECFLAGS': '-fno-strict-aliasing',
  'BASEMODLIBS': '',
  'BEOS_THREADS': 0,
  'BINDIR': '/usr/local/bin',
@@ -15,15 +15,15 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'BUILDPYTHON': 'python',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
- 'CFLAGS': '-fno-strict-aliasing -OPT:Olimit=0 -O2 -pipe -DNDEBUG -O2 -pipe   -fPIC',
+ 'CFLAGS': '-fno-strict-aliasing -O2 -pipe -DNDEBUG -O2 -pipe   -fPIC',
  'CFLAGSFORSHARED': '-fPIC',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in',
- 'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' '--srcdir=/usr/obj/ports/Python-2.7.16/Python-2.7.16' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include'",
+ 'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' '--srcdir=/pobj/Python-2.7.18/Python-2.7.18' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include'",
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python2.7',
  'COREPYTHONPATH': ':plat-openbsd6:lib-tk:lib-old',
- 'COVERAGE_INFO': '/usr/obj/ports/Python-2.7.16/Python-2.7.16/coverage.info',
- 'COVERAGE_REPORT': '/usr/obj/ports/Python-2.7.16/Python-2.7.16/lcov-report',
+ 'COVERAGE_INFO': '/pobj/Python-2.7.18/Python-2.7.18/coverage.info',
+ 'COVERAGE_REPORT': '/pobj/Python-2.7.18/Python-2.7.18/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov report"',
  'CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
  'CXX': 'c++ -pthread',
@@ -68,7 +68,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_ASM_TYPES_H': 0,
  'HAVE_ATANH': 1,
  'HAVE_ATTRIBUTE_FORMAT_PARSETUPLE': 0,
- 'HAVE_BIND_TEXTDOMAIN_CODESET': 0,
+ 'HAVE_BIND_TEXTDOMAIN_CODESET': 1,
  'HAVE_BLUETOOTH_BLUETOOTH_H': 0,
  'HAVE_BLUETOOTH_H': 0,
  'HAVE_BROKEN_NICE': 0,
@@ -185,8 +185,9 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_KILLPG': 1,
  'HAVE_KQUEUE': 1,
  'HAVE_LANGINFO_H': 1,
- 'HAVE_LARGEFILE_SUPPORT': 0,
+ 'HAVE_LARGEFILE_SUPPORT': 1,
  'HAVE_LCHFLAGS': 0,
+ 'HAVE_LCHMOD': 0,
  'HAVE_LCHOWN': 1,
  'HAVE_LGAMMA': 1,
  'HAVE_LIBDL': 0,
@@ -356,16 +357,16 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_WORKING_TZSET': 1,
  'HAVE_ZLIB_COPY': 1,
  'HAVE__GETPTY': 0,
- 'HOST_GNU_TYPE': 'x86_64-unknown-openbsd6.6',
+ 'HOST_GNU_TYPE': 'i386-unknown-openbsd6.7',
  'HURD_C_THREADS': 0,
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include /usr/local/include/python2.7 /usr/local/include/python2.7',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python2.7',
- 'INSTALL': '/usr/obj/ports/Python-2.7.16/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-2.7.16/bin/install -c -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-2.7.16/bin/install -c -s -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-2.7.16/bin/install -c -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-2.7.16/bin/install -c -m 555',
+ 'INSTALL': '/pobj/Python-2.7.18/bin/install -c',
+ 'INSTALL_DATA': '/pobj/Python-2.7.18/bin/install -c -m 644',
+ 'INSTALL_PROGRAM': '/pobj/Python-2.7.18/bin/install -c -s -m 755',
+ 'INSTALL_SCRIPT': '/pobj/Python-2.7.18/bin/install -c -m 755',
+ 'INSTALL_SHARED': '/pobj/Python-2.7.18/bin/install -c -m 555',
  'INSTSONAME': 'libpython2.7.so.0.0',
  'LDCXXSHARED': 'c++ -pthread -shared -fPIC',
  'LDFLAGS': '-L/usr/local/lib/',
@@ -385,7 +386,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'LIBPL': '/usr/local/lib/python2.7/config',
  'LIBRARY': 'libpython2.7.a',
  'LIBRARY_OBJS': '\\',
- 'LIBS': '-lpthread  -lutil',
+ 'LIBS': '-lintl -lpthread  -lutil',
  'LIBSUBDIRS': 'lib-tk lib-tk/test lib-tk/test/test_tkinter \\',
  'LINKCC': 'cc -pthread',
  'LINKFORSHARED': '-Wl,--export-dynamic',
@@ -452,7 +453,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'PYTHON_FOR_REGEN': 'python3',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
- 'PY_CFLAGS': '-fno-strict-aliasing -OPT:Olimit=0 -O2 -pipe -DNDEBUG -O2 -pipe   -fPIC -I. -IInclude -I./Include -I/usr/local/include -fPIC -DPy_BUILD_CORE',
+ 'PY_CFLAGS': '-fno-strict-aliasing -O2 -pipe -DNDEBUG -O2 -pipe   -fPIC -I. -IInclude -I./Include -I/usr/local/include -fPIC -DPy_BUILD_CORE',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
  'PY_UNICODE_TYPE': 'unsigned short',
@@ -464,12 +465,12 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'RANLIB': 'ranlib',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-2.7.16/Python-2.7.16',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/pobj/Python-2.7.18/Python-2.7.18',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SGI_ABI': '',
  'SHELL': '/bin/sh',
- 'SHLIBS': '-lpthread  -lutil',
+ 'SHLIBS': '-lintl -lpthread  -lutil',
  'SHLIB_EXT': '".so"',
  'SIGNAL_OBJS': '',
  'SIGNED_RIGHT_SHIFT_ZERO_FILLS': 0,
@@ -478,21 +479,21 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'SIZEOF_FLOAT': 4,
  'SIZEOF_FPOS_T': 8,
  'SIZEOF_INT': 4,
- 'SIZEOF_LONG': 8,
- 'SIZEOF_LONG_DOUBLE': 16,
+ 'SIZEOF_LONG': 4,
+ 'SIZEOF_LONG_DOUBLE': 12,
  'SIZEOF_LONG_LONG': 8,
  'SIZEOF_OFF_T': 8,
  'SIZEOF_PID_T': 4,
- 'SIZEOF_PTHREAD_T': 8,
+ 'SIZEOF_PTHREAD_T': 4,
  'SIZEOF_SHORT': 2,
- 'SIZEOF_SIZE_T': 8,
+ 'SIZEOF_SIZE_T': 4,
  'SIZEOF_TIME_T': 8,
- 'SIZEOF_UINTPTR_T': 8,
- 'SIZEOF_VOID_P': 8,
+ 'SIZEOF_UINTPTR_T': 4,
+ 'SIZEOF_VOID_P': 4,
  'SIZEOF_WCHAR_T': 4,
  'SIZEOF__BOOL': 1,
  'SO': '.so',
- 'SRCDIRS': 'Parser Objects Python Modules',
+ 'SRCDIRS': 'Parser Objects Python Modules Modules/_io',
  'SRC_GDB_HOOKS': './Tools/gdb/libpython.py',
  'STDC_HEADERS': 1,
  'STRICT_SYSV_CURSES': "/* Don't use ncurses extensions */",
@@ -507,7 +508,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'TESTOPTS': '-l',
  'TESTPATH': '',
  'TESTPROG': './Lib/test/regrtest.py',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-2.7.16/Python-2.7.16 ./python -Wd -3 -E -tt',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/pobj/Python-2.7.18/Python-2.7.18 ./python -Wd -3 -E -tt',
  'TESTPYTHONOPTS': '',
  'THREADOBJ': 'Python/thread.o',
  'TIME_WITH_SYS_TIME': 1,
@@ -517,7 +518,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'UNIVERSALSDK': '',
  'USE_COMPUTED_GOTOS': 0,
  'USE_TOOLBOX_OBJECT_GLUE': 0,
- 'VA_LIST_IS_ARRAY': 1,
+ 'VA_LIST_IS_ARRAY': 0,
  'VERSION': '2.7',
  'WANT_SIGFPE_HANDLER': 1,
  'WANT_WCTYPE_FUNCTIONS': 0,
@@ -530,13 +531,13 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'WITH_THREAD': 1,
  'WITH_TSC': 0,
  'WITH_VALGRIND': 0,
- 'X87_DOUBLE_ROUNDING': 0,
+ 'X87_DOUBLE_ROUNDING': 1,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-2.7.16/Python-2.7.16',
- 'abs_srcdir': '/usr/obj/ports/Python-2.7.16/Python-2.7.16',
- 'build': 'x86_64-unknown-openbsd6.6',
+ 'abs_builddir': '/pobj/Python-2.7.18/Python-2.7.18',
+ 'abs_srcdir': '/pobj/Python-2.7.18/Python-2.7.18',
+ 'build': 'i386-unknown-openbsd6.7',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
- 'host': 'x86_64-unknown-openbsd6.6',
+ 'host': 'i386-unknown-openbsd6.7',
  'prefix': '/usr/local',
  'srcdir': '.'}
