@@ -2,21 +2,19 @@
 
 Implements the Distutils 'build' command."""
 
-# This module should be kept compatible with Python 1.5.2.
-
-__revision__ = "$Id: build.py 29762 2002-11-19 13:12:28Z akuchling $"
+__revision__ = "$Id$"
 
 import sys, os
-from distutils.core import Command
+
 from distutils.util import get_platform
+from distutils.core import Command
+from distutils.errors import DistutilsOptionError
 
-
-def show_compilers ():
+def show_compilers():
     from distutils.ccompiler import show_compilers
     show_compilers()
 
-
-class build (Command):
+class build(Command):
 
     description = "build everything needed to install"
 
@@ -34,12 +32,17 @@ class build (Command):
          "build directory for scripts"),
         ('build-temp=', 't',
          "temporary build directory"),
+        ('plat-name=', 'p',
+         "platform name to build for, if supported "
+         "(default: %s)" % get_platform()),
         ('compiler=', 'c',
          "specify the compiler type"),
         ('debug', 'g',
          "compile extensions and libraries with debugging information"),
         ('force', 'f',
          "forcibly build everything (ignore file timestamps)"),
+        ('executable=', 'e',
+         "specify final destination interpreter path (build.py)"),
         ]
 
     boolean_options = ['debug', 'force']
@@ -49,7 +52,7 @@ class build (Command):
          "list available compilers", show_compilers),
         ]
 
-    def initialize_options (self):
+    def initialize_options(self):
         self.build_base = 'build'
         # these are decided only after 'build_base' has its final value
         # (unless overridden by the user or client)
@@ -59,12 +62,30 @@ class build (Command):
         self.build_temp = None
         self.build_scripts = None
         self.compiler = None
+        self.plat_name = None
         self.debug = None
         self.force = 0
+        self.executable = None
 
-    def finalize_options (self):
+    def finalize_options(self):
+        if self.plat_name is None:
+            self.plat_name = get_platform()
+        else:
+            # plat-name only supported for windows (other platforms are
+            # supported via ./configure flags, if at all).  Avoid misleading
+            # other platforms.
+            if os.name != 'nt':
+                raise DistutilsOptionError(
+                            "--plat-name only supported on Windows (try "
+                            "using './configure --help' on your platform)")
 
-        plat_specifier = ".%s-%s" % (get_platform(), sys.version[0:3])
+        plat_specifier = ".%s-%s" % (self.plat_name, sys.version[0:3])
+
+        # Make it so Python 2.x and Python 2.x with --with-pydebug don't
+        # share the same build directories. Doing so confuses the build
+        # process for C modules
+        if hasattr(sys, 'gettotalrefcount'):
+            plat_specifier += '-pydebug'
 
         # 'build_purelib' and 'build_platlib' just default to 'lib' and
         # 'lib.<plat>' under the base build directory.  We only use one of
@@ -93,11 +114,10 @@ class build (Command):
             self.build_scripts = os.path.join(self.build_base,
                                               'scripts-' + sys.version[0:3])
 
-    # finalize_options ()
-
-
-    def run (self):
+        if self.executable is None and sys.executable:
+            self.executable = os.path.normpath(sys.executable)
 
+    def run(self):
         # Run all relevant sub-commands.  This will be some subset of:
         #  - build_py      - pure Python modules
         #  - build_clib    - standalone C libraries
@@ -106,7 +126,6 @@ class build (Command):
         for cmd_name in self.get_sub_commands():
             self.run_command(cmd_name)
 
-
     # -- Predicates for the sub-command list ---------------------------
 
     def has_pure_modules (self):
@@ -121,11 +140,8 @@ class build (Command):
     def has_scripts (self):
         return self.distribution.has_scripts()
 
-
     sub_commands = [('build_py',      has_pure_modules),
                     ('build_clib',    has_c_libraries),
                     ('build_ext',     has_ext_modules),
                     ('build_scripts', has_scripts),
                    ]
-
-# class build
