@@ -3,15 +3,13 @@
 Implements the Distutils 'install_headers' command, to install C/C++ header
 files to the Python include directory."""
 
-# This module should be kept compatible with Python 2.1.
+__revision__ = "$Id$"
 
-__revision__ = "$Id: install_headers.py,v 1.11 2004/11/10 22:23:15 loewis Exp $"
-
-import os
 from distutils.core import Command
 
 
-class install_headers (Command):
+# XXX force is never used
+class install_headers(Command):
 
     description = "install C/C++ header files"
 
@@ -23,18 +21,18 @@ class install_headers (Command):
 
     boolean_options = ['force']
 
-    def initialize_options (self):
+    def initialize_options(self):
         self.install_dir = None
         self.force = 0
         self.outfiles = []
 
-    def finalize_options (self):
+    def finalize_options(self):
         self.set_undefined_options('install',
                                    ('install_headers', 'install_dir'),
                                    ('force', 'force'))
 
 
-    def run (self):
+    def run(self):
         headers = self.distribution.headers
         if not headers:
             return
@@ -44,10 +42,10 @@ class install_headers (Command):
             (out, _) = self.copy_file(header, self.install_dir)
             self.outfiles.append(out)
 
-    def get_inputs (self):
+    def get_inputs(self):
         return self.distribution.headers or []
 
-    def get_outputs (self):
+    def get_outputs(self):
         return self.outfiles
 
 # class install_headers
